# ante_build_hook.py generated by Pyrustic Manager
import sys


def get_data():
    """
    Return None or a dict with the keys:
        'script', 'target', 'app_pkg' and 'version'
    """
    items = ("script", "target", "app_pkg", "version")
    data = None
    if len(sys.argv) == len(items):
        data = {item: sys.argv[i] for i, item in enumerate(items)}
    return data


def main():
    sys.exit(0)


if __name__ == "__main__":
    main()
